@@ -2,7 +2,7 @@
 
   class.c -
 
-  $Author: akr $
+  $Author: shugo $
   created at: Tue Aug 10 15:05:44 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -13,6 +13,7 @@
 #include "ruby/signal.h"
 #include "ruby/node.h"
 #include "ruby/st.h"
+#include "vm_core.h"
 #include <ctype.h>
 
 extern st_table *rb_class_tbl;
@@ -78,10 +79,17 @@ clone_method(ID mid, NODE *body, struct clone_method_data *data)
 	st_insert(data->tbl, mid, 0);
     }
     else {
+	NODE *fbody = body->nd_body->nd_body;
+
+	if (nd_type(fbody) == RUBY_VM_METHOD_NODE) {
+	    fbody = NEW_NODE(RUBY_VM_METHOD_NODE, 0, 
+			     rb_iseq_clone((VALUE)fbody->nd_body, data->klass),
+			     0);
+	}
 	st_insert(data->tbl, mid,
 		  (st_data_t)
 		  NEW_FBODY(
-		      NEW_METHOD(body->nd_body->nd_body,
+		      NEW_METHOD(fbody,
 				 data->klass, /* TODO */
 				 body->nd_body->nd_noex),
 		      0));
@@ -102,9 +110,9 @@ rb_mod_init_copy(VALUE clone, VALUE orig)
 	ID id;
 
 	RCLASS_IV_TBL(clone) = st_copy(RCLASS_IV_TBL(orig));
-	id = rb_intern("__classpath__");
+	CONST_ID(id, "__classpath__");
 	st_delete(RCLASS_IV_TBL(clone), (st_data_t*)&id, 0);
-	id = rb_intern("__classid__");
+	CONST_ID(id, "__classid__");
 	st_delete(RCLASS_IV_TBL(clone), (st_data_t*)&id, 0);
     }
     if (RCLASS_M_TBL(orig)) {
@@ -169,10 +177,12 @@ void
 rb_singleton_class_attached(VALUE klass, VALUE obj)
 {
     if (FL_TEST(klass, FL_SINGLETON)) {
+	ID attached;
 	if (!RCLASS_IV_TBL(klass)) {
 	    RCLASS_IV_TBL(klass) = st_init_numtable();
 	}
-	st_insert(RCLASS_IV_TBL(klass), rb_intern("__attached__"), obj);
+	CONST_ID(attached, "__attached__");
+	st_insert(RCLASS_IV_TBL(klass), attached, obj);
     }
 }
 
@@ -214,8 +224,10 @@ rb_define_class_id(ID id, VALUE super)
 VALUE
 rb_class_inherited(VALUE super, VALUE klass)
 {
+    ID inherited;
     if (!super) super = rb_cObject;
-    return rb_funcall(super, rb_intern("inherited"), 1, klass);
+    CONST_ID(inherited, "inherited");
+    return rb_funcall(super, inherited, 1, klass);
 }
 
 VALUE
@@ -371,7 +383,7 @@ rb_include_module(VALUE klass, VALUE module)
     int changed = 0;
 
     rb_frozen_class_p(klass);
-    if (!OBJ_TAINTED(klass)) {
+    if (!OBJ_UNTRUSTED(klass)) {
 	rb_secure(4);
     }
     
@@ -628,9 +640,9 @@ class_instance_method_list(int argc, VALUE *argv, VALUE mod, int (*func) (ID, lo
  *       def method3()  end
  *     end
  *     
- *     A.instance_methods                #=> ["method1"]
- *     B.instance_methods(false)         #=> ["method2"]
- *     C.instance_methods(false)         #=> ["method3"]
+ *     A.instance_methods                #=> [:method1]
+ *     B.instance_methods(false)         #=> [:method2]
+ *     C.instance_methods(false)         #=> [:method3]
  *     C.instance_methods(true).length   #=> 43
  */
 
@@ -668,8 +680,8 @@ rb_class_protected_instance_methods(int argc, VALUE *argv, VALUE mod)
  *       private :method1
  *       def method2()  end
  *     end
- *     Mod.instance_methods           #=> ["method2"]
- *     Mod.private_instance_methods   #=> ["method1"]
+ *     Mod.instance_methods           #=> [:method2]
+ *     Mod.private_instance_methods   #=> [:method1]
  */
 
 VALUE
@@ -720,9 +732,9 @@ rb_class_public_instance_methods(int argc, VALUE *argv, VALUE mod)
  *       end
  *     end
  *     
- *     Single.singleton_methods    #=> ["four"]
- *     a.singleton_methods(false)  #=> ["two", "one"]
- *     a.singleton_methods         #=> ["two", "one", "three"]
+ *     Single.singleton_methods    #=> [:four]
+ *     a.singleton_methods(false)  #=> [:two, :one]
+ *     a.singleton_methods         #=> [:two, :one, :three]
  */
 
 VALUE
@@ -731,10 +743,12 @@ rb_obj_singleton_methods(int argc, VALUE *argv, VALUE obj)
     VALUE recur, ary, klass;
     st_table *list;
 
-    rb_scan_args(argc, argv, "01", &recur);
     if (argc == 0) {
 	recur = Qtrue;
     }
+    else {
+	rb_scan_args(argc, argv, "01", &recur);
+    }
     klass = CLASS_OF(obj);
     list = st_init_numtable();
     if (klass && FL_TEST(klass, FL_SINGLETON)) {
@@ -819,6 +833,12 @@ rb_singleton_class(VALUE obj)
     else {
 	FL_UNSET(klass, FL_TAINT);
     }
+    if (OBJ_UNTRUSTED(obj)) {
+	OBJ_UNTRUST(klass);
+    }
+    else {
+	FL_UNSET(klass, FL_UNTRUSTED);
+    }
     if (OBJ_FROZEN(obj)) OBJ_FREEZE(klass);
     ALLOW_INTS;
 
