@@ -127,7 +127,7 @@ class Rational
     if other.kind_of?(Rational)
       other2 = other
       if self < 0
-	return Complex.new!(self, 0) ** other
+	return Complex.__send__(:new!, self, 0) ** other
       elsif other == 0
 	return Rational(1,1)
       elsif self == 0
@@ -175,51 +175,7 @@ class Rational
 	num = 1
 	den = 1
       end
-      Rational.new!(num, den)
-    elsif other.kind_of?(Float)
-      Float(self) ** other
-    else
-      x , y = other.coerce(self)
-      x ** y
-    end
-  end
-
-  def power2(other)
-    if other.kind_of?(Rational)
-      if self < 0
-	return Complex(self, 0) ** other
-      elsif other == 0
-	return Rational(1,1)
-      elsif self == 0
-	return Rational(0,1)
-      elsif self == 1
-	return Rational(1,1)
-      end
-      
-      dem = nil
-      x = self.denominator.to_f.to_i
-      neard = self.denominator.to_f ** (1.0/other.denominator.to_f)
-      loop do
-	if (neard**other.denominator == self.denominator)
-	  dem = neaed
-	  break
-	end
-      end
-      nearn = self.numerator.to_f ** (1.0/other.denominator.to_f)
-      Rational(num,den)
-      
-    elsif other.kind_of?(Integer)
-      if other > 0
-	num = numerator ** other
-	den = denominator ** other
-      elsif other < 0
-	num = denominator ** -other
-	den = numerator ** -other
-      elsif other == 0
-	num = 1
-	den = 1
-      end
-      Rational.new!(num, den)
+      Rational(num, den)
     elsif other.kind_of?(Float)
       Float(self) ** other
     else
@@ -306,4 +262,3 @@ end
 class Complex
   Unify = true
 end
-
