@@ -2,30 +2,40 @@
 
   math.c -
 
-  $Author: shyouhei $
-  $Date: 2008-07-02 18:26:29 +0900 (Wed, 02 Jul 2008) $
+  $Author: naruse $
   created at: Tue Jan 25 14:12:56 JST 1994
 
-  Copyright (C) 1993-2003 Yukihiro Matsumoto
+  Copyright (C) 1993-2007 Yukihiro Matsumoto
 
 **********************************************************************/
 
-#include "ruby.h"
+#include "ruby/ruby.h"
 #include <math.h>
 #include <errno.h>
 
 VALUE rb_mMath;
 
-#define Need_Float(x) (x) = rb_Float(x)
+static VALUE
+to_flo(VALUE x)
+{
+    if (!rb_obj_is_kind_of(x, rb_cNumeric)) {
+	rb_raise(rb_eTypeError, "can't convert %s into Float",
+		 NIL_P(x) ? "nil" :
+		 x == Qtrue ? "true" :
+		 x == Qfalse ? "false" :
+		 rb_obj_classname(x));
+    }
+    return rb_convert_type(x, T_FLOAT, "Float", "to_f");
+}
+
+#define Need_Float(x) (x) = to_flo(x)
 #define Need_Float2(x,y) do {\
     Need_Float(x);\
     Need_Float(y);\
 } while (0)
 
 static void
-domain_check(x, msg)
-    double x;
-    char *msg;
+domain_check(double x, const char *msg)
 {
     while(1) {
 	if (errno) {
@@ -43,6 +53,24 @@ domain_check(x, msg)
     }
 }
 
+static void
+infinity_check(VALUE arg, double res, const char *msg)
+{
+    while(1) {
+	if (errno) {
+	    rb_sys_fail(msg);
+	}
+	if (isinf(res) && !isinf(RFLOAT_VALUE(arg))) {
+#if defined(EDOM)
+	    errno = EDOM;
+#elif defined(ERANGE)
+	    errno = ERANGE;
+#endif
+	    continue;
+	}
+	break;
+    }
+}
 
 /*
  *  call-seq:
@@ -53,12 +81,11 @@ domain_check(x, msg)
  *     
  */
 
-static VALUE
-math_atan2(obj, y, x)
-    VALUE obj, x, y;
+VALUE
+math_atan2(VALUE obj, VALUE y, VALUE x)
 {
     Need_Float2(y, x);
-    return rb_float_new(atan2(RFLOAT(y)->value, RFLOAT(x)->value));
+    return DOUBLE2NUM(atan2(RFLOAT_VALUE(y), RFLOAT_VALUE(x)));
 }
 
 
@@ -70,12 +97,11 @@ math_atan2(obj, y, x)
  *  -1..1.
  */
 
-static VALUE
-math_cos(obj, x)
-    VALUE obj, x;
+VALUE
+math_cos(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(cos(RFLOAT(x)->value));
+    return DOUBLE2NUM(cos(RFLOAT_VALUE(x)));
 }
 
 /*
@@ -86,13 +112,12 @@ math_cos(obj, x)
  *  -1..1.
  */
 
-static VALUE
-math_sin(obj, x)
-    VALUE obj, x;
+VALUE
+math_sin(VALUE obj, VALUE x)
 {
     Need_Float(x);
 
-    return rb_float_new(sin(RFLOAT(x)->value));
+    return DOUBLE2NUM(sin(RFLOAT_VALUE(x)));
 }
 
 
@@ -104,12 +129,11 @@ math_sin(obj, x)
  */
 
 static VALUE
-math_tan(obj, x)
-    VALUE obj, x;
+math_tan(VALUE obj, VALUE x)
 {
     Need_Float(x);
 
-    return rb_float_new(tan(RFLOAT(x)->value));
+    return DOUBLE2NUM(tan(RFLOAT_VALUE(x)));
 }
 
 /*
@@ -120,16 +144,15 @@ math_tan(obj, x)
  */
 
 static VALUE
-math_acos(obj, x)
-    VALUE obj, x;
+math_acos(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = acos(RFLOAT(x)->value);
+    d = acos(RFLOAT_VALUE(x));
     domain_check(d, "acos");
-    return rb_float_new(d);
+    return DOUBLE2NUM(d);
 }
 
 /*
@@ -140,16 +163,15 @@ math_acos(obj, x)
  */
 
 static VALUE
-math_asin(obj, x)
-    VALUE obj, x;
+math_asin(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = asin(RFLOAT(x)->value);
+    d = asin(RFLOAT_VALUE(x));
     domain_check(d, "asin");
-    return rb_float_new(d);
+    return DOUBLE2NUM(d);
 }
 
 /*
@@ -160,17 +182,15 @@ math_asin(obj, x)
  */
 
 static VALUE
-math_atan(obj, x)
-    VALUE obj, x;
+math_atan(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(atan(RFLOAT(x)->value));
+    return DOUBLE2NUM(atan(RFLOAT_VALUE(x)));
 }
 
 #ifndef HAVE_COSH
 double
-cosh(x)
-    double x;
+cosh(double x)
 {
     return (exp(x) + exp(-x)) / 2;
 }
@@ -183,19 +203,17 @@ cosh(x)
  *  Computes the hyperbolic cosine of <i>x</i> (expressed in radians).
  */
 
-static VALUE
-math_cosh(obj, x)
-    VALUE obj, x;
+VALUE
+math_cosh(VALUE obj, VALUE x)
 {
     Need_Float(x);
     
-    return rb_float_new(cosh(RFLOAT(x)->value));
+    return DOUBLE2NUM(cosh(RFLOAT_VALUE(x)));
 }
 
 #ifndef HAVE_SINH
 double
-sinh(x)
-    double x;
+sinh(double x)
 {
     return (exp(x) - exp(-x)) / 2;
 }
@@ -209,18 +227,16 @@ sinh(x)
  *  radians).
  */
 
-static VALUE
-math_sinh(obj, x)
-    VALUE obj, x;
+VALUE
+math_sinh(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(sinh(RFLOAT(x)->value));
+    return DOUBLE2NUM(sinh(RFLOAT_VALUE(x)));
 }
 
 #ifndef HAVE_TANH
 double
-tanh(x)
-    double x;
+tanh(double x)
 {
     return sinh(x) / cosh(x);
 }
@@ -235,11 +251,10 @@ tanh(x)
  */
 
 static VALUE
-math_tanh(obj, x)
-    VALUE obj, x;
+math_tanh(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(tanh(RFLOAT(x)->value));
+    return DOUBLE2NUM(tanh(RFLOAT_VALUE(x)));
 }
 
 /*
@@ -250,16 +265,15 @@ math_tanh(obj, x)
  */
 
 static VALUE
-math_acosh(obj, x)
-    VALUE obj, x;
+math_acosh(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = acosh(RFLOAT(x)->value);
+    d = acosh(RFLOAT_VALUE(x));
     domain_check(d, "acosh");
-    return rb_float_new(d);
+    return DOUBLE2NUM(d);
 }
 
 /*
@@ -270,11 +284,10 @@ math_acosh(obj, x)
  */
 
 static VALUE
-math_asinh(obj, x)
-    VALUE obj, x;
+math_asinh(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(asinh(RFLOAT(x)->value));
+    return DOUBLE2NUM(asinh(RFLOAT_VALUE(x)));
 }
 
 /*
@@ -285,16 +298,16 @@ math_asinh(obj, x)
  */
 
 static VALUE
-math_atanh(obj, x)
-    VALUE obj, x;
+math_atanh(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = atanh(RFLOAT(x)->value);
+    d = atanh(RFLOAT_VALUE(x));
     domain_check(d, "atanh");
-    return rb_float_new(d);
+    infinity_check(x, d, "atanh");
+    return DOUBLE2NUM(d);
 }
 
 /*
@@ -304,12 +317,11 @@ math_atanh(obj, x)
  *  Returns e**x.
  */
 
-static VALUE
-math_exp(obj, x)
-    VALUE obj, x;
+VALUE
+math_exp(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(exp(RFLOAT(x)->value));
+    return DOUBLE2NUM(exp(RFLOAT_VALUE(x)));
 }
 
 #if defined __CYGWIN__
@@ -324,21 +336,62 @@ math_exp(obj, x)
 /*
  *  call-seq:
  *     Math.log(numeric)    => float
+ *     Math.log(num,base)   => float
  *  
  *  Returns the natural logarithm of <i>numeric</i>.
+ *  If additional second argument is given, it will be the base
+ *  of logarithm.
+ */
+
+VALUE
+math_log(int argc, VALUE *argv)
+{
+    VALUE x, base;
+    double d;
+
+    rb_scan_args(argc, argv, "11", &x, &base);
+    Need_Float(x);
+    errno = 0;
+    d = log(RFLOAT_VALUE(x));
+    if (!NIL_P(base)) {
+	Need_Float(base);
+	d /= log(RFLOAT_VALUE(base));
+    }
+    domain_check(d, "log");
+    infinity_check(x, d, "log");
+    return DOUBLE2NUM(d);
+}
+
+#ifndef log2
+#ifndef HAVE_LOG2
+double
+log2(double x)
+{
+    return log10(x)/log10(2.0);
+}
+#else
+extern double log2(double);
+#endif
+#endif
+
+/*
+ *  call-seq:
+ *     Math.log2(numeric)    => float
+ *  
+ *  Returns the base 2 logarithm of <i>numeric</i>.
  */
 
 static VALUE
-math_log(obj, x)
-    VALUE obj, x;
+math_log2(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = log(RFLOAT(x)->value);
-    domain_check(d, "log");
-    return rb_float_new(d);
+    d = log2(RFLOAT_VALUE(x));
+    domain_check(d, "log2");
+    infinity_check(x, d, "log2");
+    return DOUBLE2NUM(d);
 }
 
 /*
@@ -349,16 +402,16 @@ math_log(obj, x)
  */
 
 static VALUE
-math_log10(obj, x)
-    VALUE obj, x;
+math_log10(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = log10(RFLOAT(x)->value);
+    d = log10(RFLOAT_VALUE(x));
     domain_check(d, "log10");
-    return rb_float_new(d);
+    infinity_check(x, d, "log10");
+    return DOUBLE2NUM(d);
 }
 
 /*
@@ -366,19 +419,74 @@ math_log10(obj, x)
  *     Math.sqrt(numeric)    => float
  *  
  *  Returns the non-negative square root of <i>numeric</i>.
+ *
+ *    0.upto(10) {|x|            
+ *      p [x, Math.sqrt(x), Math.sqrt(x)**2]
+ *    }
+ *    #=>
+ *    [0, 0.0, 0.0]
+ *    [1, 1.0, 1.0]
+ *    [2, 1.4142135623731, 2.0]
+ *    [3, 1.73205080756888, 3.0]
+ *    [4, 2.0, 4.0]
+ *    [5, 2.23606797749979, 5.0]
+ *    [6, 2.44948974278318, 6.0]
+ *    [7, 2.64575131106459, 7.0]
+ *    [8, 2.82842712474619, 8.0]
+ *    [9, 3.0, 9.0]
+ *    [10, 3.16227766016838, 10.0]
+ *
  */
 
-static VALUE
-math_sqrt(obj, x)
-    VALUE obj, x;
+VALUE
+math_sqrt(VALUE obj, VALUE x)
 {
     double d;
 
     Need_Float(x);
     errno = 0;
-    d = sqrt(RFLOAT(x)->value);
+    d = sqrt(RFLOAT_VALUE(x));
     domain_check(d, "sqrt");
-    return rb_float_new(d);
+    return DOUBLE2NUM(d);
+}
+
+/*
+ *  call-seq:
+ *     Math.cbrt(numeric)    => float
+ *  
+ *  Returns the cube root of <i>numeric</i>.
+ *
+ *    -9.upto(9) {|x|
+ *      p [x, Math.cbrt(x), Math.cbrt(x)**3]
+ *    }
+ *    #=>
+ *    [-9, -2.0800838230519, -9.0]
+ *    [-8, -2.0, -8.0]
+ *    [-7, -1.91293118277239, -7.0]
+ *    [-6, -1.81712059283214, -6.0]
+ *    [-5, -1.7099759466767, -5.0]
+ *    [-4, -1.5874010519682, -4.0]
+ *    [-3, -1.44224957030741, -3.0]
+ *    [-2, -1.25992104989487, -2.0]
+ *    [-1, -1.0, -1.0]
+ *    [0, 0.0, 0.0]
+ *    [1, 1.0, 1.0]
+ *    [2, 1.25992104989487, 2.0]
+ *    [3, 1.44224957030741, 3.0]
+ *    [4, 1.5874010519682, 4.0]
+ *    [5, 1.7099759466767, 5.0]
+ *    [6, 1.81712059283214, 6.0]
+ *    [7, 1.91293118277239, 7.0]
+ *    [8, 2.0, 8.0]
+ *    [9, 2.0800838230519, 9.0]
+ *
+ */
+
+static VALUE
+math_cbrt(VALUE obj, VALUE x)
+{
+    Need_Float(x);
+    return DOUBLE2NUM(cbrt(RFLOAT_VALUE(x)));
 }
 
 /*
@@ -394,16 +502,15 @@ math_sqrt(obj, x)
  */
 
 static VALUE
-math_frexp(obj, x)
-    VALUE obj, x;
+math_frexp(VALUE obj, VALUE x)
 {
     double d;
     int exp;
 
     Need_Float(x);
     
-    d = frexp(RFLOAT(x)->value, &exp);
-    return rb_assoc_new(rb_float_new(d), INT2NUM(exp));
+    d = frexp(RFLOAT_VALUE(x), &exp);
+    return rb_assoc_new(DOUBLE2NUM(d), INT2NUM(exp));
 }
 
 /*
@@ -417,11 +524,10 @@ math_frexp(obj, x)
  */
 
 static VALUE
-math_ldexp(obj, x, n)
-    VALUE obj, x, n;
+math_ldexp(VALUE obj, VALUE x, VALUE n)
 {
     Need_Float(x);
-    return rb_float_new(ldexp(RFLOAT(x)->value, NUM2INT(n)));
+    return DOUBLE2NUM(ldexp(RFLOAT_VALUE(x), NUM2INT(n)));
 }
 
 /*
@@ -434,12 +540,11 @@ math_ldexp(obj, x, n)
  *     Math.hypot(3, 4)   #=> 5.0
  */
 
-static VALUE
-math_hypot(obj, x, y)
-    VALUE obj, x, y;
+VALUE
+math_hypot(VALUE obj, VALUE x, VALUE y)
 {
     Need_Float2(x, y);
-    return rb_float_new(hypot(RFLOAT(x)->value, RFLOAT(y)->value));
+    return DOUBLE2NUM(hypot(RFLOAT_VALUE(x), RFLOAT_VALUE(y)));
 }
 
 /*
@@ -450,11 +555,10 @@ math_hypot(obj, x, y)
  */
 
 static VALUE
-math_erf(obj, x)
-    VALUE obj, x;
+math_erf(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(erf(RFLOAT(x)->value));
+    return DOUBLE2NUM(erf(RFLOAT_VALUE(x)));
 }
 
 /*
@@ -465,11 +569,88 @@ math_erf(obj, x)
  */
 
 static VALUE
-math_erfc(obj, x)
-    VALUE obj, x;
+math_erfc(VALUE obj, VALUE x)
 {
     Need_Float(x);
-    return rb_float_new(erfc(RFLOAT(x)->value));
+    return DOUBLE2NUM(erfc(RFLOAT_VALUE(x)));
+}
+
+/*
+ * call-seq:
+ *    Math.gamma(x)  => float
+ *
+ *  Calculates the gamma function of x.
+ *
+ *  Note that gamma(n) is same as fact(n-1) for integer n >= 0.
+ *  However gamma(n) returns float and possibly has error in calculation.
+ *
+ *   def fact(n) (1..n).inject(1) {|r,i| r*i } end
+ *   0.upto(25) {|i| p [i, Math.gamma(i+1), fact(i)] }
+ *   #=>
+ *   [0, 1.0, 1]
+ *   [1, 1.0, 1]
+ *   [2, 2.0, 2]
+ *   [3, 6.0, 6]
+ *   [4, 24.0, 24]
+ *   [5, 120.0, 120]
+ *   [6, 720.0, 720]
+ *   [7, 5040.0, 5040]
+ *   [8, 40320.0, 40320]
+ *   [9, 362880.0, 362880]
+ *   [10, 3628800.0, 3628800]
+ *   [11, 39916800.0, 39916800]
+ *   [12, 479001599.999999, 479001600]
+ *   [13, 6227020800.00001, 6227020800]
+ *   [14, 87178291199.9998, 87178291200]
+ *   [15, 1307674368000.0, 1307674368000]
+ *   [16, 20922789888000.0, 20922789888000]
+ *   [17, 3.55687428096001e+14, 355687428096000]
+ *   [18, 6.40237370572799e+15, 6402373705728000]
+ *   [19, 1.21645100408832e+17, 121645100408832000]
+ *   [20, 2.43290200817664e+18, 2432902008176640000]
+ *   [21, 5.10909421717094e+19, 51090942171709440000]
+ *   [22, 1.12400072777761e+21, 1124000727777607680000]
+ *   [23, 2.58520167388851e+22, 25852016738884976640000]
+ *   [24, 6.20448401733239e+23, 620448401733239439360000]
+ *   [25, 1.5511210043331e+25, 15511210043330985984000000]
+ *
+ */
+
+static VALUE
+math_gamma(VALUE obj, VALUE x)
+{
+    double d;
+    Need_Float(x);
+    errno = 0;
+    d = tgamma(RFLOAT_VALUE(x));
+    domain_check(d, "gamma");
+    return DOUBLE2NUM(d);
+}
+
+/*
+ * call-seq:
+ *    Math.lgamma(x)  => [float, -1 or 1]
+ *
+ *  Calculates the logarithmic gamma of x and
+ *  the sign of gamma of x.
+ *
+ *  Math.lgamma(x) is same as
+ *   [Math.log(Math.gamma(x).abs), Math.gamma(x) < 0 ? -1 : 1]
+ *  but avoid overflow by Math.gamma(x) for large x.
+ */
+
+static VALUE
+math_lgamma(VALUE obj, VALUE x)
+{
+    double d;
+    int sign;
+    VALUE v;
+    Need_Float(x);
+    errno = 0;
+    d = lgamma_r(RFLOAT_VALUE(x), &sign);
+    domain_check(d, "lgamma");
+    v = DOUBLE2NUM(d);
+    return rb_assoc_new(v, INT2FIX(sign));
 }
 
 /*
@@ -481,20 +662,20 @@ math_erfc(obj, x)
 
 
 void
-Init_Math()
+Init_Math(void)
 {
     rb_mMath = rb_define_module("Math");
 
 #ifdef M_PI
-    rb_define_const(rb_mMath, "PI", rb_float_new(M_PI));
+    rb_define_const(rb_mMath, "PI", DOUBLE2NUM(M_PI));
 #else
-    rb_define_const(rb_mMath, "PI", rb_float_new(atan(1.0)*4.0));
+    rb_define_const(rb_mMath, "PI", DOUBLE2NUM(atan(1.0)*4.0));
 #endif
 
 #ifdef M_E
-    rb_define_const(rb_mMath, "E", rb_float_new(M_E));
+    rb_define_const(rb_mMath, "E", DOUBLE2NUM(M_E));
 #else
-    rb_define_const(rb_mMath, "E", rb_float_new(exp(1.0)));
+    rb_define_const(rb_mMath, "E", DOUBLE2NUM(exp(1.0)));
 #endif
 
     rb_define_module_function(rb_mMath, "atan2", math_atan2, 2);
@@ -515,9 +696,11 @@ Init_Math()
     rb_define_module_function(rb_mMath, "atanh", math_atanh, 1);
 
     rb_define_module_function(rb_mMath, "exp", math_exp, 1);
-    rb_define_module_function(rb_mMath, "log", math_log, 1);
+    rb_define_module_function(rb_mMath, "log", math_log, -1);
+    rb_define_module_function(rb_mMath, "log2", math_log2, 1);
     rb_define_module_function(rb_mMath, "log10", math_log10, 1);
     rb_define_module_function(rb_mMath, "sqrt", math_sqrt, 1);
+    rb_define_module_function(rb_mMath, "cbrt", math_cbrt, 1);
 
     rb_define_module_function(rb_mMath, "frexp", math_frexp, 1);
     rb_define_module_function(rb_mMath, "ldexp", math_ldexp, 2);
@@ -526,4 +709,7 @@ Init_Math()
 
     rb_define_module_function(rb_mMath, "erf",  math_erf,  1);
     rb_define_module_function(rb_mMath, "erfc", math_erfc, 1);
+
+    rb_define_module_function(rb_mMath, "gamma", math_gamma, 1);
+    rb_define_module_function(rb_mMath, "lgamma", math_lgamma, 1);
 }
