@@ -2,7 +2,7 @@
 
   random.c -
 
-  $Author: akr $
+  $Author: nobu $
   created at: Fri Dec 24 16:39:21 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -20,8 +20,8 @@ The original copyright notice follows.
    This is a faster version by taking Shawn Cokus's optimization,
    Matthe Bellew's simplification, Isaku Wada's real version.
 
-   Before using, initialize the state by using init_genrand(seed) 
-   or init_by_array(init_key, key_length).
+   Before using, initialize the state by using init_genrand(mt, seed)
+   or init_by_array(mt, init_key, key_length).
 
    Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
    All rights reserved.                          
@@ -68,26 +68,31 @@ The original copyright notice follows.
 #define MIXBITS(u,v) ( ((u) & UMASK) | ((v) & LMASK) )
 #define TWIST(u,v) ((MIXBITS(u,v) >> 1) ^ ((v)&1UL ? MATRIX_A : 0UL))
 
-static unsigned long state[N]; /* the array for the state vector  */
-static int left = 1;
-static int initf = 0;
-static unsigned long *next;
+struct MT {
+    unsigned long state[N]; /* the array for the state vector  */
+    unsigned long *next;
+    int left;
+};
+
+#define genrand_initialized(mt) ((mt)->next != 0)
+#define uninit_genrand(mt) ((mt)->next = 0)
 
 /* initializes state[N] with a seed */
 static void
-init_genrand(unsigned long s)
+init_genrand(struct MT *mt, unsigned long s)
 {
     int j;
-    state[0]= s & 0xffffffffUL;
+    mt->state[0] = s & 0xffffffffUL;
     for (j=1; j<N; j++) {
-        state[j] = (1812433253UL * (state[j-1] ^ (state[j-1] >> 30)) + j); 
+        mt->state[j] = (1812433253UL * (mt->state[j-1] ^ (mt->state[j-1] >> 30)) + j);
         /* See Knuth TAOCP Vol2. 3rd Ed. P.106 for multiplier. */
         /* In the previous versions, MSBs of the seed affect   */
         /* only MSBs of the array state[].                        */
         /* 2002/01/09 modified by Makoto Matsumoto             */
-        state[j] &= 0xffffffffUL;  /* for >32 bit machines */
+        mt->state[j] &= 0xffffffffUL;  /* for >32 bit machines */
     }
-    left = 1; initf = 1;
+    mt->left = 1;
+    mt->next = mt->state + N - 1;
 }
 
 /* initialize by an array with array-length */
@@ -95,62 +100,61 @@ init_genrand(unsigned long s)
 /* key_length is its length */
 /* slight change for C++, 2004/2/26 */
 static void
-init_by_array(unsigned long init_key[], int key_length)
+init_by_array(struct MT *mt, unsigned long init_key[], int key_length)
 {
     int i, j, k;
-    init_genrand(19650218UL);
+    init_genrand(mt, 19650218UL);
     i=1; j=0;
     k = (N>key_length ? N : key_length);
     for (; k; k--) {
-        state[i] = (state[i] ^ ((state[i-1] ^ (state[i-1] >> 30)) * 1664525UL))
+        mt->state[i] = (mt->state[i] ^ ((mt->state[i-1] ^ (mt->state[i-1] >> 30)) * 1664525UL))
           + init_key[j] + j; /* non linear */
-        state[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
+        mt->state[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
         i++; j++;
-        if (i>=N) { state[0] = state[N-1]; i=1; }
+        if (i>=N) { mt->state[0] = mt->state[N-1]; i=1; }
         if (j>=key_length) j=0;
     }
     for (k=N-1; k; k--) {
-        state[i] = (state[i] ^ ((state[i-1] ^ (state[i-1] >> 30)) * 1566083941UL))
+        mt->state[i] = (mt->state[i] ^ ((mt->state[i-1] ^ (mt->state[i-1] >> 30)) * 1566083941UL))
           - i; /* non linear */
-        state[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
+        mt->state[i] &= 0xffffffffUL; /* for WORDSIZE > 32 machines */
         i++;
-        if (i>=N) { state[0] = state[N-1]; i=1; }
+        if (i>=N) { mt->state[0] = mt->state[N-1]; i=1; }
     }
 
-    state[0] = 0x80000000UL; /* MSB is 1; assuring non-zero initial array */ 
-    left = 1; initf = 1;
+    mt->state[0] = 0x80000000UL; /* MSB is 1; assuring non-zero initial array */
 }
 
 static void
-next_state(void)
+next_state(struct MT *mt)
 {
-    unsigned long *p=state;
+    unsigned long *p = mt->state;
     int j;
 
     /* if init_genrand() has not been called, */
     /* a default initial seed is used         */
-    if (initf==0) init_genrand(5489UL);
+    if (!genrand_initialized(mt)) init_genrand(mt, 5489UL);
 
-    left = N;
-    next = state;
-    
-    for (j=N-M+1; --j; p++) 
+    mt->left = N;
+    mt->next = mt->state;
+
+    for (j=N-M+1; --j; p++)
         *p = p[M] ^ TWIST(p[0], p[1]);
 
-    for (j=M; --j; p++) 
+    for (j=M; --j; p++)
         *p = p[M-N] ^ TWIST(p[0], p[1]);
 
-    *p = p[M-N] ^ TWIST(p[0], state[0]);
+    *p = p[M-N] ^ TWIST(p[0], mt->state[0]);
 }
 
 /* generates a random number on [0,0xffffffff]-interval */
 static unsigned long
-genrand_int32(void)
+genrand_int32(struct MT *mt)
 {
     unsigned long y;
 
-    if (--left == 0) next_state();
-    y = *next++;
+    if (--mt->left <= 0) next_state(mt);
+    y = *mt->next++;
 
     /* Tempering */
     y ^= (y >> 11);
@@ -163,11 +167,11 @@ genrand_int32(void)
 
 /* generates a random number on [0,1) with 53-bit resolution*/
 static double
-genrand_real(void)
-{ 
-    unsigned long a=genrand_int32()>>5, b=genrand_int32()>>6; 
-    return(a*67108864.0+b)*(1.0/9007199254740992.0); 
-} 
+genrand_real(struct MT *mt)
+{
+    unsigned long a=genrand_int32(mt)>>5, b=genrand_int32(mt)>>6;
+    return(a*67108864.0+b)*(1.0/9007199254740992.0);
+}
 /* These real versions are due to Isaku Wada, 2002/01/09 added */
 
 #undef N
@@ -187,25 +191,36 @@ genrand_real(void)
 #include <fcntl.h>
 #endif
 
+#define DEFAULT_SEED_CNT 4
+
+struct RandSeed {
+    VALUE value;
+    unsigned long initial[DEFAULT_SEED_CNT];
+};
+
+struct Random {
+    struct MT mt;
+    struct RandSeed seed;
+};
+
+static struct Random default_mt;
+
 unsigned long
 rb_genrand_int32(void)
 {
-    return genrand_int32();
+    return genrand_int32(&default_mt.mt);
 }
 
 double
 rb_genrand_real(void)
 {
-    return genrand_real();
+    return genrand_real(&default_mt.mt);
 }
 
-static VALUE saved_seed = INT2FIX(0);
-
 static VALUE
-rand_init(VALUE vseed)
+rand_init(struct MT *mt, VALUE vseed)
 {
     volatile VALUE seed;
-    VALUE old;
     long len;
     unsigned long *buf;
 
@@ -246,40 +261,28 @@ rand_init(VALUE vseed)
         len--;
     }
     if (len <= 1) {
-        init_genrand(buf[0]);
+        init_genrand(mt, buf[0]);
     }
     else {
         if (buf[len-1] == 1) /* remove leading-zero-guard */
             len--;
-        init_by_array(buf, len);
+        init_by_array(mt, buf, len);
     }
-    old = saved_seed;
-    saved_seed = seed;
-    free(buf);
-    return old;
+    xfree(buf);
+    return seed;
 }
 
-static VALUE
-random_seed(void)
+#define DEFAULT_SEED_LEN (DEFAULT_SEED_CNT * sizeof(long))
+
+static void
+fill_random_seed(unsigned long seed[DEFAULT_SEED_CNT])
 {
     static int n = 0;
     struct timeval tv;
     int fd;
     struct stat statbuf;
 
-    int seed_len;
-    BDIGIT *digits;
-    unsigned long *seed;
-    NEWOBJ(big, struct RBignum);
-    OBJSETUP(big, rb_cBignum, T_BIGNUM);
-
-    seed_len = 4 * sizeof(long);
-    RBIGNUM_SET_SIGN(big, 1);
-    rb_big_resize((VALUE)big, seed_len / SIZEOF_BDIGITS + 1);
-    digits = RBIGNUM_DIGITS(big);
-    seed = (unsigned long *)RBIGNUM_DIGITS(big);
-
-    memset(digits, 0, RBIGNUM_LEN(big) * SIZEOF_BDIGITS);
+    memset(seed, 0, DEFAULT_SEED_LEN);
 
 #ifdef S_ISCHR
     if ((fd = open("/dev/urandom", O_RDONLY
@@ -294,7 +297,7 @@ random_seed(void)
 #endif
             )) >= 0) {
         if (fstat(fd, &statbuf) == 0 && S_ISCHR(statbuf.st_mode)) {
-            read(fd, seed, seed_len);
+            read(fd, seed, DEFAULT_SEED_LEN);
         }
         close(fd);
     }
@@ -305,6 +308,20 @@ random_seed(void)
     seed[1] ^= tv.tv_sec;
     seed[2] ^= getpid() ^ (n++ << 16);
     seed[3] ^= (unsigned long)&seed;
+}
+
+static VALUE
+make_seed_value(const void *ptr)
+{
+    BDIGIT *digits;
+    NEWOBJ(big, struct RBignum);
+    OBJSETUP(big, rb_cBignum, T_BIGNUM);
+
+    RBIGNUM_SET_SIGN(big, 1);
+    rb_big_resize((VALUE)big, DEFAULT_SEED_LEN / SIZEOF_BDIGITS + 1);
+    digits = RBIGNUM_DIGITS(big);
+
+    MEMCPY((char *)RBIGNUM_DIGITS(big), ptr, char, DEFAULT_SEED_LEN);
 
     /* set leading-zero-guard if need. */
     digits[RBIGNUM_LEN(big)-1] = digits[RBIGNUM_LEN(big)-2] <= 1 ? 1 : 0;
@@ -312,6 +329,14 @@ random_seed(void)
     return rb_big_norm((VALUE)big);
 }
 
+static VALUE
+random_seed(void)
+{
+    unsigned long buf[DEFAULT_SEED_CNT];
+    fill_random_seed(buf);
+    return make_seed_value(buf);
+}
+
 /*
  *  call-seq:
  *     srand(number=0)    => old_seed
@@ -332,10 +357,14 @@ rb_f_srand(int argc, VALUE *argv, VALUE obj)
     VALUE seed, old;
 
     rb_secure(4);
-    if (rb_scan_args(argc, argv, "01", &seed) == 0) {
+    if (argc == 0) {
 	seed = random_seed();
     }
-    old = rand_init(seed);
+    else {
+	rb_scan_args(argc, argv, "01", &seed);
+    }
+    old = default_mt.seed.value;
+    default_mt.seed.value = rand_init(&default_mt.mt, seed);
 
     return old;
 }
@@ -355,7 +384,7 @@ make_mask(unsigned long x)
 }
 
 static unsigned long
-limited_rand(unsigned long limit)
+limited_rand(struct MT *mt, unsigned long limit)
 {
     unsigned long mask = make_mask(limit);
     int i;
@@ -365,7 +394,7 @@ limited_rand(unsigned long limit)
     val = 0;
     for (i = SIZEOF_LONG/4-1; 0 <= i; i--) {
         if (mask >> (i * 32)) {
-            val |= genrand_int32() << (i * 32);
+            val |= genrand_int32(mt) << (i * 32);
             val &= mask;
             if (limit < val)
                 goto retry;
@@ -375,7 +404,7 @@ limited_rand(unsigned long limit)
 }
 
 static VALUE
-limited_big_rand(struct RBignum *limit)
+limited_big_rand(struct MT *mt, struct RBignum *limit)
 {
     unsigned long mask, lim, rnd;
     struct RBignum *val;
@@ -407,7 +436,7 @@ limited_big_rand(struct RBignum *limit)
         lim = BIG_GET32(limit, i);
         mask = mask ? 0xffffffff : make_mask(lim);
         if (mask) {
-            rnd = genrand_int32() & mask;
+            rnd = genrand_int32(mt) & mask;
             if (boundary) {
                 if (lim < rnd)
                     goto retry;
@@ -448,8 +477,12 @@ rb_f_rand(int argc, VALUE *argv, VALUE obj)
 {
     VALUE vmax;
     long val, max;
+    struct MT *mt = &default_mt.mt;
 
     rb_scan_args(argc, argv, "01", &vmax);
+    if (!genrand_initialized(mt)) {
+	rand_init(mt, random_seed());
+    }
     switch (TYPE(vmax)) {
       case T_FLOAT:
 	if (RFLOAT_VALUE(vmax) <= LONG_MAX && RFLOAT_VALUE(vmax) >= LONG_MIN) {
@@ -472,10 +505,10 @@ rb_f_rand(int argc, VALUE *argv, VALUE obj)
             limit = (struct RBignum *)rb_big_minus((VALUE)limit, INT2FIX(1));
             if (FIXNUM_P((VALUE)limit)) {
                 if (FIX2LONG((VALUE)limit) == -1)
-                    return DOUBLE2NUM(genrand_real());
-                return LONG2NUM(limited_rand(FIX2LONG((VALUE)limit)));
+                    return DOUBLE2NUM(genrand_real(mt));
+                return LONG2NUM(limited_rand(mt, FIX2LONG((VALUE)limit)));
             }
-            return limited_big_rand(limit);
+            return limited_big_rand(mt, limit);
 	}
       case T_NIL:
 	max = 0;
@@ -489,18 +522,39 @@ rb_f_rand(int argc, VALUE *argv, VALUE obj)
     }
 
     if (max == 0) {
-	return DOUBLE2NUM(genrand_real());
+	return DOUBLE2NUM(genrand_real(mt));
     }
     if (max < 0) max = -max;
-    val = limited_rand(max-1);
+    val = limited_rand(mt, max-1);
     return LONG2NUM(val);
 }
 
+void
+Init_RandomSeed(void)
+{
+    fill_random_seed(default_mt.seed.initial);
+    init_by_array(&default_mt.mt, default_mt.seed.initial, DEFAULT_SEED_CNT);
+}
+
+static void
+Init_RandomSeed2(void)
+{
+    default_mt.seed.value = make_seed_value(default_mt.seed.initial);
+    memset(default_mt.seed.initial, 0, DEFAULT_SEED_LEN);
+}
+
+void
+rb_reset_random_seed(void)
+{
+    uninit_genrand(&default_mt.mt);
+    default_mt.seed.value = INT2FIX(0);
+}
+
 void
 Init_Random(void)
 {
-    rand_init(random_seed());
+    Init_RandomSeed2();
     rb_define_global_function("srand", rb_f_srand, -1);
     rb_define_global_function("rand", rb_f_rand, -1);
-    rb_global_variable(&saved_seed);
+    rb_global_variable(&default_mt.seed.value);
 }
