@@ -3,7 +3,7 @@
 
   thread_win32.c -
 
-  $Author: usa $
+  $Author: ko1 $
 
   Copyright (C) 2004-2007 Koichi Sasada
 
@@ -499,6 +499,8 @@ native_thread_join(HANDLE th)
     w32_wait_events(&th, 1, 0, 0);
 }
 
+#if USE_NATIVE_THREAD_PRIORITY
+
 static void
 native_thread_apply_priority(rb_thread_t *th)
 {
@@ -516,6 +518,8 @@ native_thread_apply_priority(rb_thread_t *th)
     SetThreadPriority(th->thread_id, priority);
 }
 
+#endif /* USE_NATIVE_THREAD_PRIORITY */
+
 static void
 ubf_handle(void *ptr)
 {
