@@ -2,8 +2,7 @@
 
   time.c -
 
-  $Author: nahi $
-  $Date: 2007-12-15 10:14:25 +0900 (Sat, 15 Dec 2007) $
+  $Author: nobu $
   created at: Tue Dec 28 14:31:59 JST 1993
 
   Copyright (C) 1993-2007 Yukihiro Matsumoto
@@ -13,6 +12,8 @@
 #include "ruby/ruby.h"
 #include <sys/types.h>
 #include <time.h>
+#include <errno.h>
+#include "ruby/encoding.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -38,7 +39,7 @@ struct time_object {
 static void
 time_free(void *tobj)
 {
-    if (tobj) free(tobj);
+    if (tobj) xfree(tobj);
 }
 
 static VALUE
@@ -59,7 +60,7 @@ static void
 time_modify(VALUE time)
 {
     rb_check_frozen(time);
-    if (!OBJ_TAINTED(time) && rb_safe_level() >= 4)
+    if (!OBJ_UNTRUSTED(time) && rb_safe_level() >= 4)
 	rb_raise(rb_eSecurityError, "Insecure: can't modify Time");
 }
 
@@ -200,6 +201,10 @@ time_timespec(VALUE num, int interval)
 	    double f, d;
 
 	    d = modf(RFLOAT_VALUE(num), &f);
+            if (d < 0) {
+                d += 1;
+                f -= 1;
+            }
 	    t.tv_sec = (time_t)f;
 	    if (f != t.tv_sec) {
 		rb_raise(rb_eRangeError, "%f out of Time range", RFLOAT_VALUE(num));
@@ -216,18 +221,24 @@ time_timespec(VALUE num, int interval)
 	break;
 
       default:
-        ary = rb_check_array_type(rb_funcall(num, id_divmod, 1, INT2FIX(1)));
-        if (NIL_P(ary)) {
+        if (rb_respond_to(num, id_divmod)) {
+            ary = rb_check_array_type(rb_funcall(num, id_divmod, 1, INT2FIX(1)));
+            if (NIL_P(ary)) {
+                goto typeerror;
+            }
+            i = rb_ary_entry(ary, 0);
+            f = rb_ary_entry(ary, 1);
+            t.tv_sec = NUM2LONG(i);
+            if (interval && t.tv_sec < 0)
+                rb_raise(rb_eArgError, "%s must be positive", tstr);
+            f = rb_funcall(f, id_mul, 1, INT2FIX(1000000000));
+            t.tv_nsec = NUM2LONG(f);
+        }
+        else {
+typeerror:
             rb_raise(rb_eTypeError, "can't convert %s into %s",
                      rb_obj_classname(num), tstr);
         }
-        i = rb_ary_entry(ary, 0);
-        f = rb_ary_entry(ary, 1);
-        t.tv_sec = NUM2LONG(i);
-	if (interval && t.tv_sec < 0)
-	    rb_raise(rb_eArgError, "%s must be positive", tstr);
-        f = rb_funcall(f, id_mul, 1, INT2FIX(1000000000));
-        t.tv_nsec = NUM2LONG(f);
 	break;
     }
     return t;
@@ -326,7 +337,7 @@ time_s_at(int argc, VALUE *argv, VALUE klass)
     return t;
 }
 
-static const char *months[] = {
+static const char months[][4] = {
     "jan", "feb", "mar", "apr", "may", "jun",
     "jul", "aug", "sep", "oct", "nov", "dec",
 };
@@ -349,7 +360,7 @@ obj2nsec(VALUE obj, long *nsec)
     if (TYPE(obj) == T_STRING) {
 	obj = rb_str_to_inum(obj, 10, Qfalse);
         *nsec = 0;
-        return NUM2LONG(obj) * 1000;
+        return NUM2LONG(obj);
     }
 
     ts = time_timespec(obj, 1);
@@ -419,7 +430,7 @@ time_arg(int argc, VALUE *argv, struct tm *tm, long *nsec)
 	    tm->tm_mon = -1;
 	    for (i=0; i<12; i++) {
 		if (RSTRING_LEN(s) == 3 &&
-		    strcasecmp(months[i], RSTRING_PTR(v[1])) == 0) {
+		    STRCASECMP(months[i], RSTRING_PTR(s)) == 0) {
 		    tm->tm_mon = i;
 		    break;
 		}
@@ -472,6 +483,18 @@ static VALUE time_gmtime(VALUE);
 static VALUE time_localtime(VALUE);
 static VALUE time_get_tm(VALUE, int);
 
+#ifdef HAVE_GMTIME_R
+#define IF_HAVE_GMTIME_R(x) x
+#define ASCTIME(tm, buf) asctime_r(tm, buf)
+#define GMTIME(tm, result) gmtime_r(tm, &result)
+#define LOCALTIME(tm, result) localtime_r(tm, &result)
+#else
+#define IF_HAVE_GMTIME_R(x) 	/* nothing */
+#define ASCTIME(tm, buf) asctime(tm)
+#define GMTIME(tm, result) gmtime(tm)
+#define LOCALTIME(tm, result) localtime(tm)
+#endif
+
 static int
 leap_year_p(long y)
 {
@@ -483,7 +506,7 @@ leap_year_p(long y)
 static time_t
 timegm_noleapsecond(struct tm *tm)
 {
-    static int common_year_yday_offset[] = {
+    static const int common_year_yday_offset[] = {
 	-1,
 	-1 + 31,
 	-1 + 31 + 28,
@@ -498,7 +521,7 @@ timegm_noleapsecond(struct tm *tm)
 	-1 + 31 + 28 + 31 + 30 + 31 + 30 + 31 + 31 + 30 + 31 + 30
 	  /* 1    2    3    4    5    6    7    8    9    10   11 */
     };
-    static int leap_year_yday_offset[] = {
+    static const int leap_year_yday_offset[] = {
 	-1,
 	-1 + 31,
 	-1 + 31 + 29,
@@ -571,6 +594,8 @@ search_time_t(struct tm *tptr, int utc_p)
     struct tm *tm, tm_lo, tm_hi;
     int d, have_guess;
     int find_dst;
+    IF_HAVE_GMTIME_R(struct tm result);
+#define GUESS(p) (utc_p ? GMTIME(p, result) : LOCALTIME(p, result))
 
     find_dst = 0 < tptr->tm_isdst;
 
@@ -584,7 +609,7 @@ search_time_t(struct tm *tptr, int utc_p)
 	       ~(time_t)0;
 
     guess = timegm_noleapsecond(tptr);
-    tm = (utc_p ? gmtime : localtime)(&guess);
+    tm = GUESS(&guess);
     if (tm) {
 	d = tmcmp(tptr, tm);
 	if (d == 0) return guess;
@@ -596,8 +621,7 @@ search_time_t(struct tm *tptr, int utc_p)
 	    guess_lo = guess;
 	    guess += 24 * 60 * 60;
 	}
-	if (guess_lo < guess && guess < guess_hi &&
-	    (tm = (utc_p ? gmtime : localtime)(&guess)) != NULL) {
+	if (guess_lo < guess && guess < guess_hi && (tm = GUESS(&guess)) != NULL) {
 	    d = tmcmp(tptr, tm);
 	    if (d == 0) return guess;
 	    if (d < 0)
@@ -607,14 +631,14 @@ search_time_t(struct tm *tptr, int utc_p)
 	}
     }
 
-    tm = (utc_p ? gmtime : localtime)(&guess_lo);
+    tm = GUESS(&guess_lo);
     if (!tm) goto error;
     d = tmcmp(tptr, tm);
     if (d < 0) goto out_of_range;
     if (d == 0) return guess_lo;
     tm_lo = *tm;
 
-    tm = (utc_p ? gmtime : localtime)(&guess_hi);
+    tm = GUESS(&guess_hi);
     if (!tm) goto error;
     d = tmcmp(tptr, tm);
     if (d > 0) goto out_of_range;
@@ -711,7 +735,7 @@ search_time_t(struct tm *tptr, int utc_p)
 	    range = 0;
 	}
 
-	tm = (utc_p ? gmtime : localtime)(&guess);
+	tm = GUESS(&guess);
 	if (!tm) goto error;
 	have_guess = 0;
 	
@@ -742,7 +766,7 @@ search_time_t(struct tm *tptr, int utc_p)
 		time_t guess2;
 		if (find_dst) {
 		    guess2 = guess - 2 * 60 * 60;
-		    tm = localtime(&guess2);
+		    tm = LOCALTIME(&guess2, result);
 		    if (tm) {
 			if (tptr->tm_hour != (tm->tm_hour + 2) % 24 ||
 			    tptr->tm_min != tm->tm_min ||
@@ -754,7 +778,7 @@ search_time_t(struct tm *tptr, int utc_p)
 			    if (tptr->tm_mday != tm->tm_mday)
 				guess2 += 24 * 60 * 60;
 			    if (guess != guess2) {
-				tm = localtime(&guess2);
+				tm = LOCALTIME(&guess2, result);
 				if (tmcmp(tptr, tm) == 0) {
 				    if (guess < guess2)
 					return guess;
@@ -767,7 +791,7 @@ search_time_t(struct tm *tptr, int utc_p)
 		}
 		else {
 		    guess2 = guess + 2 * 60 * 60;
-		    tm = localtime(&guess2);
+		    tm = LOCALTIME(&guess2, result);
 		    if (tm) {
 			if ((tptr->tm_hour + 2) % 24 != tm->tm_hour ||
 			    tptr->tm_min != tm->tm_min ||
@@ -779,7 +803,7 @@ search_time_t(struct tm *tptr, int utc_p)
 			    if (tptr->tm_mday != tm->tm_mday)
 				guess2 -= 24 * 60 * 60;
 			    if (guess != guess2) {
-				tm = localtime(&guess2);
+				tm = LOCALTIME(&guess2, result);
 				if (tmcmp(tptr, tm) == 0) {
 				    if (guess < guess2)
 					return guess2;
@@ -822,14 +846,19 @@ static time_t
 make_time_t(struct tm *tptr, int utc_p)
 {
     time_t t;
-    struct tm *tmp, buf;
+#ifdef NEGATIVE_TIME_T
+    struct tm *tmp;
+#endif
+    struct tm buf;
+    IF_HAVE_GMTIME_R(struct tm result);
+
     buf = *tptr;
     if (utc_p) {
 #if defined(HAVE_TIMEGM)
 	if ((t = timegm(&buf)) != -1)
 	    return t;
 #ifdef NEGATIVE_TIME_T
-	if ((tmp = gmtime(&t)) &&
+	if ((tmp = GMTIME(&t, result)) &&
 	    tptr->tm_year == tmp->tm_year &&
 	    tptr->tm_mon == tmp->tm_mon &&
 	    tptr->tm_mday == tmp->tm_mday &&
@@ -847,7 +876,7 @@ make_time_t(struct tm *tptr, int utc_p)
 	if ((t = mktime(&buf)) != -1)
 	    return t;
 #ifdef NEGATIVE_TIME_T
-	if ((tmp = localtime(&t)) &&
+	if ((tmp = LOCALTIME(&t, result)) &&
 	    tptr->tm_year == tmp->tm_year &&
 	    tptr->tm_mon == tmp->tm_mon &&
 	    tptr->tm_mday == tmp->tm_mday &&
@@ -1040,11 +1069,8 @@ time_nsec(VALUE time)
 /*
  *  call-seq:
  *     time <=> other_time => -1, 0, +1 
- *     time <=> numeric    => -1, 0, +1
  *  
- *  Comparison---Compares <i>time</i> with <i>other_time</i> or with
- *  <i>numeric</i>, which is the number of seconds (possibly
- *  fractional) since the Epoch.
+ *  Comparison---Compares <i>time</i> with <i>other_time</i>.
  *     
  *     t = Time.now       #=> 2007-11-19 08:12:12 -0600
  *     t2 = t + 2592000   #=> 2007-12-19 08:12:12 -0600
@@ -1076,8 +1102,18 @@ time_cmp(VALUE time1, VALUE time2)
 	if (tobj1->ts.tv_sec > tobj2->ts.tv_sec) return INT2FIX(1);
 	return INT2FIX(-1);
     }
+    else {
+	VALUE cmp;
+	int n;
 
-    return Qnil;
+	cmp = rb_funcall(time2, rb_intern("<=>"), 1, time1);
+	if (NIL_P(cmp)) return Qnil;
+
+	n = rb_cmpint(cmp, time1, time2);
+	if (n == 0) return INT2FIX(0);
+	if (n > 0) return INT2FIX(1);
+	return INT2FIX(-1);
+    }
 }
 
 /*
@@ -1196,6 +1232,7 @@ time_localtime(VALUE time)
     struct time_object *tobj;
     struct tm *tm_tmp;
     time_t t;
+    IF_HAVE_GMTIME_R(struct tm result);
 
     GetTimeval(time, tobj);
     if (!tobj->gmt) {
@@ -1206,7 +1243,7 @@ time_localtime(VALUE time)
 	time_modify(time);
     }
     t = tobj->ts.tv_sec;
-    tm_tmp = localtime(&t);
+    tm_tmp = LOCALTIME(&t, result);
     if (!tm_tmp)
 	rb_raise(rb_eArgError, "localtime error");
     tobj->tm = *tm_tmp;
@@ -1239,6 +1276,7 @@ time_gmtime(VALUE time)
     struct time_object *tobj;
     struct tm *tm_tmp;
     time_t t;
+    IF_HAVE_GMTIME_R(struct tm result);
 
     GetTimeval(time, tobj);
     if (tobj->gmt) {
@@ -1249,7 +1287,7 @@ time_gmtime(VALUE time)
 	time_modify(time);
     }
     t = tobj->ts.tv_sec;
-    tm_tmp = gmtime(&t);
+    tm_tmp = GMTIME(&t, result);
     if (!tm_tmp)
 	rb_raise(rb_eArgError, "gmtime error");
     tobj->tm = *tm_tmp;
@@ -1321,17 +1359,22 @@ time_asctime(VALUE time)
 {
     struct time_object *tobj;
     char *s;
+    IF_HAVE_GMTIME_R(char buf[32]);
 
     GetTimeval(time, tobj);
     if (tobj->tm_got == 0) {
 	time_get_tm(time, tobj->gmt);
     }
-    s = asctime(&tobj->tm);
+    s = ASCTIME(&tobj->tm, buf);
     if (s[24] == '\n') s[24] = '\0';
 
     return rb_str_new2(s);
 }
 
+size_t
+rb_strftime(char *s, size_t maxsize, const char *format,
+	    const struct tm *timeptr, const struct timespec *ts, int gmt);
+
 /*
  *  call-seq:
  *     time.inspect => string
@@ -1360,24 +1403,12 @@ time_to_s(VALUE time)
 	time_get_tm(time, tobj->gmt);
     }
     if (tobj->gmt == 1) {
-	len = strftime(buf, 128, "%Y-%m-%d %H:%M:%S UTC", &tobj->tm);
+	len = rb_strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S UTC",
+			  &tobj->tm, &tobj->ts, tobj->gmt);
     }
     else {
-	long off;
-	char sign = '+';
-#if defined(HAVE_STRUCT_TM_TM_GMTOFF)
-	off = tobj->tm.tm_gmtoff;
-#else
-	VALUE tmp = time_utc_offset(time);
-	off = NUM2INT(tmp);
-#endif
-	if (off < 0) {
-	    sign = '-';
-	    off = -off;
-	}
-	len = strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S ", &tobj->tm);
-        len += snprintf(buf+len, sizeof(buf)-len, "%c%02d%02d", sign,
-                        (int)(off/3600), (int)(off%3600/60));
+	len = rb_strftime(buf, sizeof(buf), "%Y-%m-%d %H:%M:%S %z",
+			  &tobj->tm, &tobj->ts, tobj->gmt);
     }
     return rb_str_new(buf, len);
 }
@@ -1833,7 +1864,7 @@ time_yday(VALUE time)
  *  Returns <code>true</code> if <i>time</i> occurs during Daylight
  *  Saving Time in its time zone.
  *     
- *   CST6CDT:
+ *   # CST6CDT:
  *     Time.local(2000, 1, 1).zone    #=> "CST"
  *     Time.local(2000, 1, 1).isdst   #=> false
  *     Time.local(2000, 1, 1).dst?    #=> false
@@ -1841,7 +1872,7 @@ time_yday(VALUE time)
  *     Time.local(2000, 7, 1).isdst   #=> true
  *     Time.local(2000, 7, 1).dst?    #=> true
  *
- *   Asia/Tokyo:
+ *   # Asia/Tokyo:
  *     Time.local(2000, 1, 1).zone    #=> "JST"
  *     Time.local(2000, 1, 1).isdst   #=> false
  *     Time.local(2000, 1, 1).dst?    #=> false
@@ -1897,7 +1928,8 @@ time_zone(VALUE time)
 #elif defined(HAVE_TZNAME) && defined(HAVE_DAYLIGHT)
     return rb_str_new2(tzname[daylight && tobj->tm.tm_isdst]);
 #else
-    len = strftime(buf, 64, "%Z", &tobj->tm);
+    len = rb_strftime(buf, sizeof(buf), "%Z",
+		      &tobj->tm, &tobj->ts, tobj->gmt);
     return rb_str_new(buf, len);
 #endif
 }
@@ -1937,9 +1969,10 @@ time_utc_offset(VALUE time)
 	struct tm *u, *l;
 	time_t t;
 	long off;
+	IF_HAVE_GMTIME_R(struct tm result);
 	l = &tobj->tm;
 	t = tobj->ts.tv_sec;
-	u = gmtime(&t);
+	u = GMTIME(&t, result);
 	if (!u)
 	    rb_raise(rb_eArgError, "gmtime error");
 	if (l->tm_year != u->tm_year)
@@ -1997,7 +2030,8 @@ time_to_a(VALUE time)
 
 #define SMALLBUF 100
 static int
-rb_strftime(char **buf, const char *format, struct tm *time)
+rb_strftime_alloc(char **buf, const char *format,
+		  struct tm *time, struct timespec *ts, int gmt)
 {
     int size, len, flen;
 
@@ -2006,12 +2040,13 @@ rb_strftime(char **buf, const char *format, struct tm *time)
     if (flen == 0) {
 	return 0;
     }
-    len = strftime(*buf, SMALLBUF, format, time);
-    if (len != 0 || **buf == '\0') return len;
+    errno = 0;
+    len = rb_strftime(*buf, SMALLBUF, format, time, ts, gmt);
+    if (len != 0 || (**buf == '\0' && errno != ERANGE)) return len;
     for (size=1024; ; size*=2) {
 	*buf = xmalloc(size);
 	(*buf)[0] = '\0';
-	len = strftime(*buf, size, format, time);
+	len = rb_strftime(*buf, size, format, time, ts, gmt);
 	/*
 	 * buflen can be zero EITHER because there's not enough
 	 * room in the string, or because the control command
@@ -2020,7 +2055,7 @@ rb_strftime(char **buf, const char *format, struct tm *time)
 	 * format string, it's not failing for lack of room.
 	 */
 	if (len > 0 || size >= 1024 * flen) return len;
-	free(*buf);
+	xfree(*buf);
     }
     /* not reached */
 }
@@ -2040,11 +2075,17 @@ rb_strftime(char **buf, const char *format, struct tm *time)
  *    %B - The  full  month  name (``January'')
  *    %c - The preferred local date and time representation
  *    %d - Day of the month (01..31)
+ *    %F - Equivalent to %Y-%m-%d (the ISO 8601 date format)
  *    %H - Hour of the day, 24-hour clock (00..23)
  *    %I - Hour of the day, 12-hour clock (01..12)
  *    %j - Day of the year (001..366)
+ *    %L - Millisecond of the second (000..999)
  *    %m - Month of the year (01..12)
  *    %M - Minute of the hour (00..59)
+ *    %N - Fractional seconds digits, default is 9 digits (nanosecond)
+ *            %3N  millisecond (3 digits)
+ *            %6N  microsecond (6 digits)
+ *            %9N  nanosecond (9 digits)
  *    %p - Meridian indicator (``AM''  or  ``PM'')
  *    %S - Second of the minute (00..60)
  *    %U - Week  number  of the current year,
@@ -2069,6 +2110,7 @@ rb_strftime(char **buf, const char *format, struct tm *time)
 static VALUE
 time_strftime(VALUE time, VALUE format)
 {
+    void rb_enc_copy(VALUE, VALUE);
     struct time_object *tobj;
     char buffer[SMALLBUF], *buf = buffer;
     const char *fmt;
@@ -2080,6 +2122,9 @@ time_strftime(VALUE time, VALUE format)
 	time_get_tm(time, tobj->gmt);
     }
     StringValue(format);
+    if (!rb_enc_str_asciicompat_p(format)) {
+	rb_raise(rb_eArgError, "format should have ASCII compatible encoding");
+    }
     format = rb_str_new4(format);
     fmt = RSTRING_PTR(format);
     len = RSTRING_LEN(format);
@@ -2092,11 +2137,11 @@ time_strftime(VALUE time, VALUE format)
 
 	str = rb_str_new(0, 0);
 	while (p < pe) {
-	    len = rb_strftime(&buf, p, &tobj->tm);
+	    len = rb_strftime_alloc(&buf, p, &tobj->tm, &tobj->ts, tobj->gmt);
 	    rb_str_cat(str, buf, len);
 	    p += strlen(p);
 	    if (buf != buffer) {
-		free(buf);
+		xfree(buf);
 		buf = buffer;
 	    }
 	    for (fmt = p; p < pe && !*p; ++p);
@@ -2105,10 +2150,12 @@ time_strftime(VALUE time, VALUE format)
 	return str;
     }
     else {
-	len = rb_strftime(&buf, RSTRING_PTR(format), &tobj->tm);
+	len = rb_strftime_alloc(&buf, RSTRING_PTR(format),
+			       	&tobj->tm, &tobj->ts, tobj->gmt);
     }
     str = rb_str_new(buf, len);
-    if (buf != buffer) free(buf);
+    if (buf != buffer) xfree(buf);
+    rb_enc_copy(str, format);
     return str;
 }
 
@@ -2127,11 +2174,12 @@ time_mdump(VALUE time)
     int nsec;
     int i;
     VALUE str;
+    IF_HAVE_GMTIME_R(struct tm result);
 
     GetTimeval(time, tobj);
 
     t = tobj->ts.tv_sec;
-    tm = gmtime(&t);
+    tm = GMTIME(&t, result);
 
     if ((tm->tm_year & 0xffff) != tm->tm_year)
         rb_raise(rb_eArgError, "year too big to marshal: %ld", (long)tm->tm_year);
@@ -2309,7 +2357,7 @@ time_load(VALUE klass, VALUE str)
  *  stored internally as the number of seconds and nanoseconds since
  *  the <em>Epoch</em>, January 1, 1970 00:00 UTC. On some operating
  *  systems, this offset is allowed to be negative. Also see the
- *  library modules <code>Date</code> and <code>ParseDate</code>. The
+ *  library modules <code>Date</code>. The
  *  <code>Time</code> class treats GMT (Greenwich Mean Time) and UTC
  *  (Coordinated Universal Time)<em>[Yes, UTC really does stand for
  *  Coordinated Universal Time. There was a committee involved.]</em>
@@ -2325,6 +2373,9 @@ time_load(VALUE klass, VALUE str)
 void
 Init_Time(void)
 {
+#undef rb_intern
+#define rb_intern(str) rb_intern_const(str)
+
     id_divmod = rb_intern("divmod");
     id_mul = rb_intern("*");
     id_submicro = rb_intern("submicro");
